package eu.ztsh.garmin.mapbox

import kotlinx.coroutines.delay
import kotlinx.coroutines.runBlocking
import java.util.concurrent.CopyOnWriteArraySet
import java.util.concurrent.atomic.AtomicReference

class NavigationStatusControl {

    private val stateObservers = CopyOnWriteArraySet<NavigationStatusObserver>()
    private val current = AtomicReference(NavigationStatus.IDLE)

    fun registerObserver(observer: NavigationStatusObserver) {
        stateObservers.add(observer)
        observer.onNavigationStatusChanged(current.get())
    }

    fun unregisterObserver(observer: NavigationStatusObserver) {
        stateObservers.remove(observer)
    }

    fun sendEvent(status: NavigationStatus) {
        current.set(status)
        stateObservers.forEach {
            it.onNavigationStatusChanged(status)
        }
        if (status == NavigationStatus.FINISHED || status == NavigationStatus.CANCELED) {
            // TODO: lifecyclescope?
            runBlocking {
                delay(1000)
                sendEvent(NavigationStatus.IDLE)
            }
        }
    }

}

enum class NavigationStatus {
    IDLE,
    STARTED,
    FINISHED,
    CANCELED
}

fun interface NavigationStatusObserver {

    fun onNavigationStatusChanged(navigationStatus: NavigationStatus)

}